@@ -1,6 +1,7 @@
 #' Show columns that have no variation
 #' @description
-#' This function is called by \code{\link{show_constant}}. The app in question
+#' This function is called by \code{\link[=show_constant]{show_constant()}}.
+#' The app in question
 #' is already specified there. I must admit that I found the idea to this
 #' code somewhere on Stack Overflow but cannot remember where.
 #' @param df Data frame.
@@ -17,12 +18,16 @@ constant_col <- function(
   if (!is.na(value)) {
 
     # Check for NA values
-    logi_vec <- sapply(df, function(df) all(!is.na(df) & df == value))
+    logi_vec <- vapply(df,
+                       function(df) all(!is.na(df) & df == value),
+                       FUN.VALUE = logical(1L))
 
   } else if (is.na(value)) {
 
     # Check for NA
-    logi_vec <- sapply(df, function(df) all(is.na(df)))
+    logi_vec <- vapply(df,
+                       function(df) all(is.na(df)),
+                       FUN.VALUE = logical(1L))
   }
 
   # Get the names of all constant columns
